package adrian.master.knowledgewar.gameconcepts;

import java.util.LinkedList;
import java.util.List;

import org.json.JSONArray;
import org.json.JSONException;

/**
 * Battle is the class that keeps all information about an ongoing battle. 
 * It contains the involved players, the battle-identifier, the questions, the score and it also keeps track of how much money a player uses during a duel.
 * The questions are stored in a List, and the Battle keeps track of the current question itself so the Battle-Activity on has to ask for the next question.
 * @author Adrian
 *
 */
public class Battle {
	
	Player mPlayer1, mPlayer2;
	List<Question> mQuestionList = new LinkedList<Question>();
	private boolean[] results;
	private String mBattleNr;
	private int mScore;
	private int mCurrentQuestion = -1;
	public int mSize;
	private int mTotalWeaponValue;
	
	/**
	 * Simple constructor that creates the object and stores the identifier so the game can tell the server what battle is going on.
	 * @param nr
	 */
	public Battle(String nr){
		mBattleNr=nr;
		mTotalWeaponValue = 0;
	}
	
	/**
	 * Creates the list of questions. Called from the BattleActivity after fetching the questionset from the server.
	 * @param array JSONArray fetched from the server, containing the questions and alternatives.
	 */
	public void populateQuestionList(JSONArray array){
		
		for(int i=0;i<array.length(); i++){
			try {
				Question temp = new Question(array.getJSONObject(i));
				mQuestionList.add(temp);
			} catch (JSONException e) {
				// TODO Handle JSONException in Battle.populateQuestionList
				e.printStackTrace();
			}
		}
		mSize = mQuestionList.size();
		results = new boolean[mSize];
	}

	public String getBattleNr(){
		return mBattleNr;
	}

	/**
	 * Used to determine if the current question is the last one.
	 * @return Whether or not this is the final question.
	 */
	public boolean hasNext(){
		return !(mCurrentQuestion==mQuestionList.size()-1);
	}

	/**
	 * Called when a question is answered, and the game needs the next question to display.
	 * @return
	 */
	public boolean next(){
		mCurrentQuestion++;
		if(mCurrentQuestion>mQuestionList.size()-1){
			return false;
		}
		return true;
	}
	
	public Question getCurrentQuestion(){
		return mQuestionList.get(mCurrentQuestion);
	}

	/**
	 * Returns whether or not the question answered a specific question correctly.
	 * @param nr
	 * @return
	 */
	public boolean isQuestionCorrect(int nr){
		return results[nr];
	}

	/**
	 * Returns a specific Question-object.
	 * @param nr
	 * @return
	 */
	public Question getQuestion(int nr){
		return mQuestionList.get(nr);
	}

	/**
	 * Returns the question-text from a specific question
	 * @param nr
	 * @return
	 */
	public String getQuestionNr(int nr){
		return mQuestionList.get(nr).getQuestion();
	}

	/**
	 * When a player uses a weapon, the cost of that weapon is added to a total that is subtracted from the profits at the end of the battle.
	 * @param cost
	 */
	public void addToWeaponValue(int cost){
		mTotalWeaponValue+=cost;
	}

	public int getWeaponValue(){
		return mTotalWeaponValue;
	}

	/**
	 * Add score from a single question to the total
	 * @param score Score based on the amount of time the player took to answer.
	 */
	public void addScore(int score){
		this.mScore+=score;
	}

	public int getScore(){
		return mScore;
	}

	/**
	 * Keeps track of whether or not the player answered the question correctly. 
	 * @param result
	 */
	public void updateResult(boolean result){
		this.results[mCurrentQuestion]=result;
	}
	
	/**
	 * If the player forfeits, the battle is set to end, and the score to 0.
	 */
	public void forfeitBattle(){
		mScore=0;
		mCurrentQuestion=mQuestionList.size()-1;
	}

}
