package adrian.master.knowledgewar.gameconcepts;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import adrian.master.knowledgewar.R;
import android.content.Context;
import android.util.Log;

/**
 * Factory-class that creates templates of all the Buildings. Stats for buildings are read from an xml-file on startup, and this class then creates 
 * a List containing all the Buildings, so that the game can quickly get copy of any of the templates.
 * @author Adrian
 *
 */
public class BuildingFactory {
	
	private static ArrayList<Building> buildingTemplates;
	
	/**
	 * Called when creating the Player-object based on the Area-string fetched from the server. A player's area is represented by a String that contains
	 * information about each building, and this method converts that information into actual buildings that can be used by the game.
	 * @param parsableInputString Part of the area-String fetched from the server. Follows format [Building type]:[Building rank]:[x-coordinate],[y-coordinate].
	 * @return
	 */
	public static Building constructBuilding(String parsableInputString){
		Building result = parseInputString(parsableInputString);
		return result;
	}
	
	/**
	 * Called when the Player builds a new building in his tribe.
	 * @param x x-coordinate of the building.
	 * @param y y-coordinate of the building.
	 * @param type Type of Building.
	 * @return A finished building.
	 */
	public static Building constructBuilding(int x, int y, String type){
		//Method only gets called when building a NEW building, so rank is always 1.
		Building result = getNewBuildingFromTemplate(x, y, type, 1);
		return result;
	}
	
	/**
	 * Returns all ranks of one type of Building. Used to give an overview of what a specific building does if the player levels it up.
	 * @param type The type of the Buildings requested.
	 * @return List containing all Buildings of that type.
	 */
	public static ArrayList<Building> getAllBuildingsOfType(String type){
		ArrayList<Building> result = new ArrayList<Building>();
		int i =1;
		for(Building b: buildingTemplates){
			if(b.getType().equals(type) && b.getRank()==i){
				result.add(getNewBuildingFromTemplate(0, 0, type, i));
				i++;
			}
		}
		
		return result;
	}
	
	/**
	 * Returns a building of one higher rank than the input-building. Used when levelling up a building.
	 * @param building
	 * @return
	 */
	public static Building getUpgradedBuilding(Building building){
		Building result=null;
		if(building==null){
			Log.v("Knowledgewar getupgradedbuilding","paramter-building is null");
		}
		for(Building b:buildingTemplates){
			if(b.getType().equals(building.getType()) && b.getRank() == building.getRank()+1){
				result = getNewBuildingFromTemplate(building.getX(), building.getY(), building.getType(), building.getRank()+1);
			}
		}
		
		return result;
	}
	
	/**
	 * Creates a copy of one Building-template, that can be used by the game without having to worry about accidentally destroying or altering it.
	 * @param x
	 * @param y
	 * @param type
	 * @param rank
	 * @return
	 */
	private static Building getNewBuildingFromTemplate(int x, int y, String type, int rank){
		Iterator<Building> it = buildingTemplates.iterator();
		Building result=null;
		while(it.hasNext()){
			Building b = it.next();
			if(b.getType().equals(type) && b.getRank()==rank){
				result = Building.getNewBuilding(b);
				result.setX(x);
				result.setY(y);
				break;
			}
		}
		return result;
	}
	
	/**
	 * Parses the xml-file containing information about all the Buildings, and creates a List containing templates of all the buildings.
	 * @param context
	 */
	public static void parseXML(Context context){
		buildingTemplates = new ArrayList<Building>();
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		Document dom;
		try {
			InputStream catDoc = context.getResources().openRawResource(R.raw.buildings);
			DocumentBuilder db = dbf.newDocumentBuilder();
			dom = db.parse(catDoc, null);
			
			Element doc = dom.getDocumentElement();
			NodeList nl = doc.getElementsByTagName("Building");
			
			if(nl!=null){
				for(int i=0;i<nl.getLength();i++){
					Element el = (Element)nl.item(i);
					constructBuildingTemplate(el);
				}
			}
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SAXException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	/**
	 * Used in parsing the xml-file.
	 * @param el
	 */
	private static void constructBuildingTemplate(Element el){
		String type = getNodeValue(el, "Type");
		String name = getNodeValue(el, "Name");
		String description = getNodeValue(el, "Description");
		NodeList nl = el.getElementsByTagName("Rank");
		if(nl != null){
			for(int i=0;i<nl.getLength();i++){
				Element ele = (Element)nl.item(i);
				int number = Integer.parseInt(getNodeValue(ele, "Number"));
				int value = Integer.parseInt(getNodeValue(ele, "Value"));
				int health = Integer.parseInt(getNodeValue(ele, "Health"));
				boolean global = getNodeValue(ele, "Global").equals("true");
				float goldBonus = Float.parseFloat(getNodeValue(ele, "GoldBonus"));
				int damageBonus = Integer.parseInt(getNodeValue(ele, "DamageBonus"));
				int healthBonus = Integer.parseInt(getNodeValue(ele, "HealthBonus"));
				float goldPenalty = Float.parseFloat(getNodeValue(ele, "GoldPenalty"));
				int upgradeCost = Integer.parseInt(getNodeValue(ele, "UpgradeCost"));			
				Building b = new Building(type, name, number, value, health, global, 
						goldBonus, damageBonus, healthBonus, goldPenalty, upgradeCost, description);
				buildingTemplates.add(b);
			}
		}
	}
	
	/**
	 * Used in parsing the xml-file.
	 * @param el
	 * @param tagName
	 * @return
	 */
	private static String getNodeValue(Element el, String tagName){
		String value="";
		
		NodeList nl = el.getElementsByTagName(tagName);
		if(nl!=null){
			Element ele = (Element)nl.item(0);
			value = ele.getFirstChild().getNodeValue();
		}
		
		return value;
	}
	
	/**
	 * Parses a part of the area-String that belongs to a Player, and returns a Building based on the information in that String.
	 * @param input
	 * @return
	 */
	private static Building parseInputString(String input){
		String[] substrings = new String[3];
		
		for(int i=0; i<substrings.length;i++ ){
			int index = input.indexOf(':');
			if(index!=-1){
				substrings[i] = input.substring(0, index);				
			}else{
				substrings[i] = input;
			}
			input = input.substring(index+1);
		}
		
		String type = substrings[0];
		int rank = Integer.parseInt(substrings[1]);
		
		int indexOfCoordinateSeparator = substrings[2].indexOf(',');
		int x=Integer.parseInt(substrings[2].substring(0,indexOfCoordinateSeparator));
		int y=Integer.parseInt(substrings[2].substring(indexOfCoordinateSeparator+1));
		
		return getNewBuildingFromTemplate(x, y, type, rank);
	}
	
}
