package adrian.master.knowledgewar.gameconcepts;

import java.util.ArrayList;
import java.util.Iterator;

import adrian.master.knowledgewar.logic.Constants;

/**
 * Class that contains information about a Player. Either the Player themselves, or opponents during battles.
 * @author Adrian
 *
 */
public class Player {

	private String name;
	private int x, y;
	private int gold;
	private String area;
	private ArrayList<Building> buildings;
	private boolean mBusy=false;

	public float totalGoldModifier=0;
	public float totalExpModifier=0;
	public float totalOpponentGoldModifier=0;
	public float totalOpponentExpModifier=0;
	public int totalDamageBonus=0;
	public int totalHealthBonus=0;
	public int totalBuildingValue=0;

	public float[] xRange={0,0}, yRange={0,0};


	/**
	 * Constructor used for the Player-object for the player themselves
	 * @param name
	 * @param area
	 * @param gold
	 * @param xp
	 */
	public Player(String name, String area, int gold){
		buildings = new ArrayList<Building>();
		this.name=name;
		this.area=area;
		this.gold=gold;
		parseAreaBuildings(area);
	}

	/**
	 * Constructor used for the Player-objects for the opponents, where the gold and xp are unavailable 
	 * @param name
	 * @param area
	 */
	public Player(String name, String area, boolean busy){
		buildings = new ArrayList<Building>();
		this.name=name;
		this.area=area;
		this.mBusy=busy;
		parseAreaBuildings(area);
	}


	public String getName(){
		return name;
	}

	public boolean isBusy(){
		return mBusy;
	}

	public String getArea(){
		return area;
	}

	/**
	 * Returns the sum of the ranks of all the Player's buildings. Used to give an indication of the strength of the player.
	 * @return
	 */
	public int getRank(){
		int result = 0;
		for(Building b: buildings){
			result+=b.getRank();
		}
		return result;
	}

	public int getGold(){
		return gold;
	}

	public ArrayList<Building> getBuildings(){
		return buildings;
	}

	/**
	 * Adds up the bonuses granted by the buildings in the involved area.
	 */
	public void calculateBonuses(){
		totalGoldModifier=0;
		totalDamageBonus=0;
		totalExpModifier=0;
		totalHealthBonus=0;
		totalOpponentGoldModifier=0;
		totalOpponentExpModifier=0;
		totalBuildingValue=0;

		for(Building b : buildings){
			if(b.isGlobal() || ((b.getX()>=xRange[0] && b.getX()<=xRange[1]) && (b.getY()>=yRange[0] && b.getY()<=yRange[1]))){
				totalGoldModifier+=b.getGoldBonus();
				totalDamageBonus+=b.getDamageBonus();
				totalHealthBonus+=b.getHealthBonus();
				totalOpponentGoldModifier-=b.getGoldPenalty();
				totalBuildingValue+=b.getValue();
			}
		}
	}

	/**
	 * Used when determining available weapons. Checks if the player has a certain building, and if they do, returns the 
	 * rank of that building. If the player does not have the building, this returns -1
	 * @param buildingType
	 * @return
	 */
	public int hasSpecificBuildingAndRank(String buildingType){
		int result = -1;
		for(Building b: buildings){
			if(b.getType().equals(buildingType)){
				result = b.getRank();
			}
		}
		return result;
	}


	/**
	 * Parses the entire area-String fetched from the server, and creates buildings and such based on that String.
	 * @param areaBuildings
	 */
	private void parseAreaBuildings(String areaBuildings){

		ArrayList<String> strings = new ArrayList<String>();
		String area2 = areaBuildings;
		int index=0;
		do{
			index=area2.indexOf(';');
			String temp="";
			if(index!=-1){
				temp = area2.substring(0, index);
				area2 = area2.substring(index+1);
			}else{
				temp = area2;
			}
			strings.add(temp);
		}while(index!=-1);


		int indexOfSeparator = strings.get(0).indexOf('x');
		x=Integer.parseInt(strings.get(0).substring(0, indexOfSeparator));
		y=Integer.parseInt(strings.get(0).substring(indexOfSeparator+1));


		for(int i=1; i<strings.size();i++){

			Building temp = BuildingFactory.constructBuilding(strings.get(i));
			buildings.add(temp);
		}

	}

	/**
	 * Called when a Player places a new Building in their tribe. 
	 * @param building
	 * @param px
	 * @param py
	 * @return
	 */
//	public boolean addNewBuilding(Building building, int px, int py){
//		boolean isTileOccupied = false;
//		if(gold>=building.getValue()){
//			for(Building b: buildings){
//				if(b.getX()==px && b.getY()==py){
//					isTileOccupied=true;
//				}
//			}
//
//			if(!isTileOccupied && 
//					(px==0 && py==0 || px==4 && py==4 || px==4 && py==0 || px==0 && py==4 || px==2 && py==2)){
//				building.setX(px);
//				building.setY(py);
//				payGold(building.getValue());
//				buildings.add(building);
//				return true;			
//			}			
//		}
//
//		return false;
//	}

	public boolean isTileOccupied(int px, int py){
		boolean isTileOccupied = false;
		for(Building b: buildings){
			if(b.getX()==px && b.getY()==py){
				isTileOccupied=true;
			}
		}

		return isTileOccupied;
	}

	public Building getBuildingAtCoordinates(int px, int py){
		int coordX = px/Constants.TILE_SIZE;
		int coordY = py/Constants.TILE_SIZE;
		for(Building b: buildings){
			if(b.getX()==coordX && b.getY()==coordY){
				return b;
			}
		}
		return null;
	}

	public boolean canUpgradeBuilding(Building newBuilding){
		return (newBuilding != null && hasEnoughGold(newBuilding.getUpgradeCost()));
	}

	public int getX(){
		return x;
	}

	public int getY(){
		return y;
	}

	/**
	 * Creates a correctly formatted area-String that can be uploaded to the server.
	 * @return
	 */
	public String getAreaString(){
		String result = "";
		result+=x+"x"+y;
		for(Building b: buildings){
			result+=b.toString();
		}
		return result;
	}

	private boolean hasEnoughGold(int gold){
		return (this.gold-gold)>=0;
	}


	public void payGold(int gold){
		this.gold-=gold;
	}

}
